use core::time;
use std::thread;

use easymacros::xwrap::{display, key, screen};

use x11::xlib;

use clap::Parser;

/// Macro recording module for easymacros. Outputs are partially compatible with xmacro.
#[derive(Parser, Debug)]
#[clap(author, version, about, long_about = None)]
struct Args {
    /// The file to record the macro to. Defaults to writing to stdout.
    #[clap(value_parser, value_name = "output_file", value_hint = clap::ValueHint::FilePath)]
    output_file: Option<std::path::PathBuf>,
    /// Display to run the macro on. This uses the $DISPLAY environment variable by default.
    #[clap(short = 'D', long)]
    display: Option<String>,
    /// Max Delay in milliseconds for macro delays
    #[clap(short, long)]
    max_delay: Option<u64>,
    /// Allow delay capturing in recording output. If this flag is set, the program will ignore the max_delay.
    #[clap(short, long)]
    ignore_delay_capturing: bool,
}

fn main() -> anyhow::Result<()> {
    let args = Args::parse();

    let mut display =
        display::Display::open(args.display.clone()).expect("should be able to open to display");

    let stop_key = get_stop_key(&mut display);

    dbg!(stop_key);

    Ok(())
}

fn get_stop_key(display: &mut display::Display) -> key::Key {
    let root = display.default_root_window();

    display
        .grab_keyboard(
            root,
            false,
            display::GrabMode::Sync,
            display::GrabMode::Sync,
            xlib::CurrentTime,
        )
        .expect("keyboard should be available to be grabbed");

    println!("Press the key you want to use to stop recording the macro.");

    let stop_key = loop {
        display.allow_events(display::EventMode::SyncPointer, xlib::CurrentTime);
    };

    thread::sleep(time::Duration::from_secs(3));

    todo!()
}
