use x11::xlib;

use super::{display, error};

use anyhow::Result;

pub use x11::keysym;

/// Stores a keycode
#[derive(Debug)]
pub struct Key {
    code: u32,
}

impl Key {
    /// Creates a `Key` from a keycode and its related display connection.
    pub fn from_code(code: xlib::KeyCode, display: &display::Display) -> Result<Self> {
        let valid_range = display.keycodes()?;

        if valid_range.contains(&code.into()) {
            Ok(Key { code: code.into() })
        } else {
            Err(error::XError::InvalidKeycodeError(code, valid_range).into())
        }
    }

    /// Creates a `Key` from a keysym by converting it into a code internally.
    pub fn from_keysym(keysym: xlib::KeySym, display: &display::Display) -> Result<Option<Self>> {
        let valid_range = display.keycodes()?;

        let code = unsafe { xlib::XKeysymToKeycode(display.ptr, keysym) };

        Ok(if code == 0 {
            None
        } else {
            Some(Key { code: code.into() })
        })
    }
}
